#version 330 core

#define MAX_POINT_LIGHTS 100

uniform vec3 cameraPosition;

// Ambient light
#define AMBIENT_LIGHT vec3(0.1, 0.1, 0.1)

// Directional light
uniform struct DirectionalLight
{
	vec3 direction;
	vec3 color;
};
uniform DirectionalLight directionalLight;

// Point light
uniform int pointLightCount;
uniform vec3 pointLightPositions[MAX_POINT_LIGHTS];	
uniform vec3 pointLightColors[MAX_POINT_LIGHTS];
uniform vec3 pointLightAttenuations[MAX_POINT_LIGHTS];

// Texture
uniform sampler2D texture0;

// Material
struct Material
{
	vec3 specular;
	float shininess;
};
uniform Material material;

// Vertex attributes
in vec3 FragPos;
in vec3 Normal;
in vec2 TexCoords;

// Output
out vec4 FragColor;

// Function prototypes
vec3 CalcDirectionalLight(vec3 normal, vec3 viewDir, vec3 lightDir, vec3 lightColor);
vec3 CalcPointLight(vec3 normal, vec3 viewDir, vec3 lightPos, vec3 lightColor, vec3 attenuation);

void main()
{
	vec3 objectColor = texture(texture0, TexCoords).rgb;
	vec3 normal = normalize(Normal);
	vec3 result = vec3(0.0, 0.0, 0.0);

	// Add ambient component
	result += AMBIENT_LIGHT * objectColor;

	// Add directional light
	vec3 viewDir = normalize(cameraPosition - FragPos);
	result += CalcDirectionalLight(normal, viewDir, directionalLight.direction, directionalLight.color) * objectColor;

	// Add point lights
	for (int i = 0; i < pointLightCount; i++)
	{
		result += CalcPointLight(normal, viewDir, pointLightPositions[i], pointLightColors[i], pointLightAttenuations[i]) * objectColor;
	}

	FragColor = vec4(result, 1.0);

	// Gamma correction
	FragColor.rgb = pow(FragColor.rgb, vec3(1.0 / 2.2));
}

vec3 CalcDirectionalLight(vec3 normal, vec3 viewDir, vec3 lightDir, vec3 lightColor)
{
	vec3 lightVec = normalize(-lightDir);
	float diff = max(dot(normal, lightVec), 0.0);
	vec3 diffuse = diff * lightColor;

	vec3 reflectDir = reflect(-lightVec, normal);
	float spec = pow(max(dot(viewDir, reflectDir), 0.0), material.shininess);
	vec3 specular = spec * lightColor * material.specular;

	return (diffuse + specular);
}

vec3 CalcPointLight(vec3 normal, vec3 viewDir, vec3 lightPos, vec3 lightColor, vec3 lightAtt)
{
	vec3 lightDir = normalize(lightPos - FragPos);
	float diff = max(dot(normal, lightDir), 0.0);
	vec3 diffuse = diff * lightColor;
	float distance = length(lightPos - FragPos);
	float attenuation = 1.0 / (lightAtt.x + lightAtt.y * distance + lightAtt.z * distance * distance);
	diffuse *= attenuation;

	vec3 reflectDir = reflect(-lightDir, normal);
	float spec = pow(max(dot(viewDir, reflectDir), 0.0), material.shininess);
	vec3 specular = spec * lightColor * attenuation * material.specular;

	return (diffuse + specular);
}